INCLUDE gba_common.ld

SECTIONS {
    /* VMA LAYOUT:
     *
     * ROM:
     * * boot code
     * * slow code
     * * rodata
     * IWRAM:
     * * fast code
     * * fast rodata
     * * fast data
     * * fast bss
     * * STACK
     * EWRAM:
     * * code
     * * ewram rodata
     * * data
     * * bss
     * * HEAP
     */

    /* ROM stuff */

    . = ORIGIN(romws0);
    __text_start = .;
    .text.crt0 :
    {
        KEEP (*(.text.crt0 .text.startup))
        . = ALIGN(4);
    } >romws0 =0xff

    .text.slow :
    {
        *(.text.rom .text.slow .text.unlikely)
        . = ALIGN(4);
    } >romws0 =0xff

    .rodata :
    {
        *(.rodata.rom .rodata)
        . = ALIGN(4);
    } >romws0 =0xff

    /* IWRAM stuff */

    __iwram_lma = .;
    .iwram ORIGIN(iwram) : AT (__iwram_lma)
    {
        __iwram_startv = ABSOLUTE(.);
        *(.text.isr .text.iwram .text.hot .text.arm)
        . = ALIGN(2);
        *(.iwram*)
        . = ALIGN(2);
        *(.rodata.iwram)
        . = ALIGN(4);
    } >iwram = 0xff

    __iwdata_lma = __iwram_lma + SIZEOF(.iwram);
    .data.iwram ALIGN(4) : AT (__iwdata_lma)
    {
        *(.data.iwram)
        . = ALIGN(4);
    } >iwram =0xff

    __iwram_endl = __iwdata_lma + SIZEOF(.data.iwram);
    __iwram_endv = ABSOLUTE(.); /*ORIGIN(iwram) + SIZEOF(.iwram) + SIZEOF(.data.iwram);*/

    .bss.iwram ALIGN(4) (NOLOAD) :
    {
        PROVIDE(__iwbss_start = ABSOLUTE(.));
        *(.bss.iwram* .sbss*)
        . = ALIGN(4);
    } AT>iwram

    .stack ALIGN(4) (NOLOAD) :
    {
        __sp_max = ABSOLUTE(.);
        ASSERT(__sp_max < __sp_sys - __min_stack_size, "No space left for stack!");
    } AT>iwram

    /* EWRAM stuff */

    __ewram_lma = __iwdata_lma + SIZEOF(.data.iwram);
    .ewram ORIGIN(ewram) : AT (__ewram_lma)
    {
        __ewram_startv = ABSOLUTE(.);
        *(.text.ewram .text .text.thumb)
        . = ALIGN(2);
        *(.ewram)
        . = ALIGN(2);
        *(.rodata.ewram)
        . = ALIGN(2);
    } >ewram =0xff

    __ewdata_lma = __ewram_lma + SIZEOF(.ewram);
    .data ALIGN(2) : AT(__ewdata_lma)
    {
        *(.data .data.ewram)
        . = ALIGN(2);
    } >ewram =0xff

    __ewram_endl = __ewdata_lma + SIZEOF(.data);
    __ewram_endv = ABSOLUTE(.); /*ORIGIN(ewram) + SIZEOF(.ewram) + SIZEOF(.data);*/

    .bss ALIGN(2) (NOLOAD) :
    {
        PROVIDE(__ewbss_start = ABSOLUTE(.));
        *(.bss .bss.ewram)
        . = ALIGN(2);
    } AT>ewram

    .heap ALIGN(4) (NOLOAD) :
    {
        __heap_start = ABSOLUTE(.);
        PROVIDE(__heap_start = __heap_start);
        /*. = __heap_end;*/
        ASSERT(__heap_start < __heap_end - __min_heap_size, "No space left for heap!");
    }

    __rom_end = __ewdata_lma + SIZEOF(.data);
}

